using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Windows;
using System.Threading;

using FlickrNet;
using System.IO;
using System.Collections.ObjectModel;

namespace M2xdam.VistaFlickrUploader
{
    /// <summary>
    /// Helper class for Flickr admin tasks.
    /// </summary>
    class FlickrHelper
    {
        /// <summary>
        /// The full authorization class that gives the current user permission for Flickr administration tasks.
        /// </summary>
        private Auth _authorization;

        /// <summary>
        /// The Flickr API object.
        /// </summary>
        private Flickr _flickr;

        /// <summary>
        /// Structure with upload thread status values.
        /// </summary>
        private UploadThreadStatus _uploadThreadStatus;

        public delegate void UploadEventHandler(object sender, UploadThreadStatus e);
        public event UploadEventHandler OnPictureUploadProgress;
        public event UploadEventHandler OnUploadException;
        public event UploadEventHandler OnUploadComplete;

        /// <summary>
        /// The authorization token that gives the current user permission for Flickr administration tasks.
        /// </summary>
        private void SaveFlickrAuthToken(string token)
        {
            Properties.Settings.Default.FlickrAuthToken = token;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Returns the upload status for the current user.
        /// </summary>
        public UserStatus CurrentStatus
        {
            get
            {
                return _flickr.PeopleGetUploadStatus();
            }
        }

        /// <summary>
        /// Returns the currently active Flickr user.
        /// </summary>
        public Person CurrentUser
        {
            get
            {
                return _flickr.PeopleGetInfo(this._authorization.User.UserId);
            }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public FlickrHelper(bool allowAuthorizationProcedure)
        {
            _uploadThreadStatus = new UploadThreadStatus();

            ConnectToFlickrServer(allowAuthorizationProcedure);
        }

        /// <summary>
        /// Returns the list of photosets for the current user.
        /// </summary>
        /// <returns></returns>
        public List<Photoset> GetPhotoSets()
        {
            List<Photoset> list = new List<Photoset>(_flickr.PhotosetsGetList());
            list.Sort(delegate(Photoset a, Photoset b) { return a.Title.CompareTo(b.Title); });
            return list;
        }

        /// <summary>
        /// Creates a new Flickr object and validates its authorization.
        /// </summary>
        private void ConnectToFlickrServer(bool allowAuthorizationProcedure)
        {
            Flickr.FlushCache();
            _flickr = new Flickr(Properties.Settings.Default.FlickrApiKey,
                                 Properties.Settings.Default.FlickrSharedSecret,
                                 Properties.Settings.Default.FlickrAuthToken);

            try
            {
                _authorization = _flickr.AuthCheckToken(_flickr.AuthToken);
            }
            catch
            {
                if (allowAuthorizationProcedure)
                {
                    GetFlickrAuthorization();
                }
                else
                {
                    _authorization = null;
                    throw;
                }
            }

            //List<Photoset> list = GetPhotoSets();
            //foreach (Photoset set in list)
            //{
            //    if (set.Title.StartsWith("Test"))
            //        _flickr.PhotosetsDelete(set.PhotosetId);
            //}
            //for (int i = 0; i < 100; i++)
            //{
            //    _flickr.PhotosetsCreate("TestSet" + i.ToString(), "502143029");
            //}
        }

        /// <summary>
        /// Tells the uploading thread to stop.
        /// </summary>
        public void CancelUpload()
        {
            _uploadThreadStatus.CancelUpload = true;
        }

        /// <summary>
        /// Uploads a list of pictures asynchronously.
        /// </summary>
        /// <param name="pictures">The list of pictures that should be uploaded.</param>
        public void UploadPicturesAsync(Collection<PictureItem> pictures, bool forFamily, bool forFriends, bool forEveryone, string photoSetId)
        {
            _uploadThreadStatus.CancelUpload = false;
            _uploadThreadStatus.ForFamily    = forFamily;
            _uploadThreadStatus.ForFriends   = forFriends;
            _uploadThreadStatus.ForEveryone  = forEveryone;
            _uploadThreadStatus.PhotoSetId   = photoSetId;

            // Create a list with shallow copies of the specified pictures
            _uploadThreadStatus.Pictures = new List<PictureItem>(pictures.Count);
            foreach (PictureItem picture in pictures)
            {
                _uploadThreadStatus.Pictures.Add(picture.ShallowCopy());
            }

            // Start a new thread in which the uploads will take place
            Thread thread = new Thread(new ThreadStart(this.DoUpload));
            thread.Start();
        }

        /// <summary>
        /// Threading method that uploads the list of pictures.
        /// </summary>
        private void DoUpload()
        {
            _uploadThreadStatus.StartStopwatch();

            // Upload pictures one by one
            bool success = true;
            _uploadThreadStatus.CurrentPictureIndex = 0;
            foreach (PictureItem picture in _uploadThreadStatus.Pictures)
            {
                success = UploadPictureSync(picture, 
                                    _uploadThreadStatus.ForFamily, 
                                    _uploadThreadStatus.ForFriends, 
                                    _uploadThreadStatus.ForEveryone, 
                                    _uploadThreadStatus.PhotoSetId);

                if (!success)
                {
                    break;
                }
                _uploadThreadStatus.CurrentPictureIndex++;
            }

            _uploadThreadStatus.StopStopwatch();

            // Broadcast end of batch
            FireUploadEvent(OnUploadComplete, 100);
        }

        private void FireUploadEvent(UploadEventHandler eventHandler, byte picturePercentageComplete)
        {
            _uploadThreadStatus.PicturePercentageComplete = picturePercentageComplete;
            if (eventHandler != null)
            {
                eventHandler(this, _uploadThreadStatus);
            }
        }

        /// <summary>
        /// Upload a single picture synchronously.
        /// </summary>
        /// <param name="picture">The picture to upload.</param>
        /// <returns>The Flickr photo ID.</returns>
        private bool UploadPictureSync(PictureItem picture, bool forFamily, bool forFriends, bool forEveryone, string photoSetId)
        {
            _uploadThreadStatus.Exception = null;
            try
            {
                // Check possible cancelation
                if (_uploadThreadStatus.CancelUpload)
                {
                    throw new ApplicationException("Upload canceled by user.");
                }
                
                // Broadcast start of upload
                FireUploadEvent(OnPictureUploadProgress, 0);

                // Upload picture
                _flickr.OnUploadProgress += _flickr_OnPictureUploadProgress;
                picture.FlickrPhotoId = _flickr.UploadPicture(picture.FileInfo.FullName, picture.Title, picture.Comment, picture.TagListForFlickr, forEveryone, forFamily, forFriends);

                if (photoSetId != null)
                {
                    _flickr.PhotosetsAddPhoto(photoSetId, picture.FlickrPhotoId);
                }

                // Broadcast end of upload
                FireUploadEvent(OnPictureUploadProgress, 100);
                return true;
            }
            catch (Exception ex)
            {
                _uploadThreadStatus.Exception = ex;

                // Broadcast upload exception
                FireUploadEvent(OnUploadException, 0);
                return false;
            }
        }

        /// <summary>
        /// The upload event that is triggered by the Flickr API component.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _flickr_OnPictureUploadProgress(object sender, FlickrNet.UploadProgressEventArgs e)
        {
            // Calculate percentage complete
            byte percentage = (byte)((95 * e.Bytes) / _uploadThreadStatus.CurrentPicture.FileInfo.Length);

            // Extra safety check, although it shouldn't be necessary
            if (percentage > 95)
            {
                percentage = 95;
            }

            // Broadcast upload progress
            FireUploadEvent(OnPictureUploadProgress, percentage);
        }

        /// <summary>
        /// Requests a new authorization token.
        /// </summary>
        private void GetFlickrAuthorization()
        {
            try
            {
                _flickr.AuthToken = null;
                string frob = _flickr.AuthGetFrob();
                string url  = _flickr.AuthCalcUrl(frob, AuthLevel.Write);

                // Open web browser in which the user should authorize this application
                System.Diagnostics.Process.Start(url);

                // Wait for the user to confirm that the authorization procedure is complete
                while (MessageBox.Show(Application.Current.MainWindow, Properties.Messages.MsgFlickrAuthorization.Replace("\\n", "\n"), Application.Current.MainWindow.Title, MessageBoxButton.YesNo, MessageBoxImage.Information, MessageBoxResult.No) != MessageBoxResult.Yes)
                { }

                // Remember the token for ever
                SaveFlickrAuthToken(_flickr.AuthGetToken(frob).Token);

                // Reconnect to Flickr (otherwise the first photo's will be lost)
                ConnectToFlickrServer(false);
            }
            catch (FlickrException ex)
            {
                throw new ApplicationException(Properties.Messages.MsgNotAuthorized + " (" + ex.Message + ")");
            }
        }
    }
}
