using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace M2xdam.VistaFlickrUploader
{
    /// <summary>
    /// Wrapper class that contains a picture and associated information fields.
    /// </summary>
    class PictureItem : IDisposable
    {
        private Guid _guid;
        private FileStream _imageFileStream;
        private BitmapFrame _bitmapFrame;

        private FileInfo _fileInfo;
        private string _title;
        private string _comment;
        private List<string> _keywords;

        private string _flickrPhotoId;

        public Guid Guid
        {
            get { return _guid; }
        }

        public BitmapSource Thumbnail
        {
            get
            {
                return _bitmapFrame.Thumbnail == null ? _bitmapFrame : _bitmapFrame.Thumbnail;
            }
        }
        public BitmapMetadata Metadata
        {
            get
            {
                return (BitmapMetadata)_bitmapFrame.Metadata;
            }
        }
        public string DisplaySize
        {
            get
            {
                return string.Format("{0}x{1}", _bitmapFrame.PixelWidth, _bitmapFrame.PixelHeight);
            }
        }

        public FileInfo FileInfo
        {
            get
            {
                return _fileInfo;
            }
        }
        public string TagListForDisplay
        {
            get
            {
                StringBuilder tags = new StringBuilder();
                foreach (string keyword in _keywords)
                {
                    tags.AppendFormat("{0}, ", keyword);
                }
                if (tags.Length >= 2)
                {
                    tags.Length -= 2;
                }
                return tags.ToString();
            }
        }
        public string TagListForFlickr
        {
            get
            {
                StringBuilder tags = new StringBuilder();
                foreach (string keyword in _keywords)
                {
                    tags.AppendFormat("{1}{0}{1} ", keyword, keyword.Contains(" ")?"\"":"");
                }
                if (tags.Length >= 1)
                {
                    tags.Length -= 1;
                }
                return tags.ToString();
            }
        }
        public string Title
        {
            get { return _title; }
            set { _title = value; }
        }
        public string Comment
        {
            get { return _comment; }
            set { _comment = value; }
        }
        public List<string> Keywords
        {
            get { return _keywords; }
            set { _keywords = value; }
        }

        public string FlickrPhotoId
        {
            get { return _flickrPhotoId; }
            set { _flickrPhotoId = value; }
        }

        private PictureItem()
        {
            _imageFileStream = null;
            _bitmapFrame = null;
        }
        public PictureItem(string fileName)
        {
            _guid = new Guid();

            // Remember the file stream so we can close and release the handle in Dispose
            _imageFileStream = new FileStream(fileName, FileMode.Open, FileAccess.Read);

            // Disable caching to prevent excessive memory use
            _bitmapFrame = BitmapFrame.Create(_imageFileStream, BitmapCreateOptions.None, BitmapCacheOption.None);

            _fileInfo = new FileInfo(fileName);
            _title    = Path.GetFileNameWithoutExtension(fileName);
            _comment  = "";
            _keywords = new List<string>(0);
            if (Metadata != null)
            {
                try
                {
                    if (!string.IsNullOrEmpty(Metadata.Title))
                    {
                        this._title = Metadata.Title;
                    }
                }
                catch (NotSupportedException) { }
                try
                {
                    this._comment = Metadata.Comment;
                }
                catch (NotSupportedException) { }
                try
                {
                    if (Metadata.Keywords != null)
                    {
                        this._keywords = new List<string>(Metadata.Keywords);
                    }
                }
                catch (NotSupportedException) { }
            }

            if (Properties.Settings.Default.TagSeparators.Count > 0)
            {
                string[] tagSeparators = new string[Properties.Settings.Default.TagSeparators.Count];
                Properties.Settings.Default.TagSeparators.CopyTo(tagSeparators, 0);

                List<string> splittedList = new List<string>();
                foreach (string keyword in _keywords)
                {
                    string[] parts = keyword.Split(tagSeparators, StringSplitOptions.RemoveEmptyEntries);
                    splittedList.AddRange(parts);
                }
                _keywords = splittedList;
            }

            if (Properties.Settings.Default.TagsToExclude.Count > 0)
            {
                string[] tagsToExclude = new string[Properties.Settings.Default.TagsToExclude.Count];
                Properties.Settings.Default.TagsToExclude.CopyTo(tagsToExclude, 0);
                _keywords.RemoveAll(delegate(string keyword)
                    {
                        return Array.Exists<string>(tagsToExclude, delegate(string exclusion)
                            {
                                return string.Compare(keyword, exclusion, true) == 0;
                            });
                    });
            }
        }

        public PictureItem ShallowCopy()
        {
            PictureItem picture = new PictureItem();
            picture._guid     = this._guid;
            picture._fileInfo = this._fileInfo;
            picture._title    = this._title;
            picture._comment  = this._comment;
            picture._keywords = new List<string>(this._keywords);
            return picture;
        }

        public void Dispose()
        {
            if (_imageFileStream != null)
            {
                _imageFileStream.Dispose();
            }
        }
    }
}
